/*
   CheckList 
   Copyright (C) 1999 Jarek Woloszyn (yossa@pld.org.pl)

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

/* This goofed-up box whacked into shape by Elliot Lee <sopwith@cuc.edu>
   (from the original listbox by Erik Troan <ewt@redhat.com>)
   and contributed to newt for use under the LGPL license.
   Copyright (C) 1996, 1997 Elliot Lee */

#include <slang/slang.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <newt.h>
#include "checklist.h"
#include "newt_pr.h"


/* Linked list of items in the listbox */
struct items {
    char *text;
    const void *data;
    unsigned char isSelected;
    struct items *next;
    char isChecked;
    char *ptrChecked;
};

/* Holds all the relevant information for this listbox */
struct checklist {
    newtComponent sb;		/* Scrollbar on right side of listbox */
    int curWidth;		/* size of text w/o scrollbar or border */
    int curHeight;		/* size of text w/o border */
    int sbAdjust;
    int bdxAdjust, bdyAdjust;
    int numItems, numSelected;
    int userHasSetWidth;
    int currItem, startShowItem;	/* startShowItem is the first item displayed
					   on the screen */
    int isActive;		/* If we handle key events all the time, it seems
				   to do things even when they are supposed to be for
				   another button/whatever */
    struct items *boxItems;
    int grow;
    int mctype;
    enum newtChecklistStatus status;
    int flags;			/* flags for this listbox, right now just
				   NEWT_FLAG_RETURNEXIT */
};

static void checklistDraw(newtComponent co);
static void checklistDestroy(newtComponent co);
static struct eventResult checklistEvent(newtComponent co, struct event ev);
static void newtChecklistRealSetCurrent(newtComponent co);
static void checklistPlace(newtComponent co, int newLeft, int newTop);
static inline void updateWidth(newtComponent co, struct checklist *li,
			       int maxField);
static void checklistMapped(newtComponent co, int isMapped);

static struct componentOps checklistOps =
{
    checklistDraw,
    checklistEvent,
    checklistDestroy,
    checklistPlace,
    checklistMapped,
};

static void checklistMapped(newtComponent co, int isMapped)
{
    struct checklist *li = co->data;

    co->isMapped = isMapped;
    if (li->sb)
	li->sb->ops->mapped(li->sb, isMapped);
}

static void checklistPlace(newtComponent co, int newLeft, int newTop)
{
    struct checklist *li = co->data;

    co->top = newTop;
    co->left = newLeft;

    if (li->sb)
	li->sb->ops->place(li->sb, co->left + co->width - li->bdxAdjust - 1,
			   co->top);
}

newtComponent newtChecklist(int left, int top, int height, int flags)
{
    newtComponent co, sb;
    struct checklist *li;

    if (!(co = malloc(sizeof(*co))))
	return NULL;

    if (!(li = malloc(sizeof(struct checklist)))) {
	free(co);
	return NULL;
    }
    li->boxItems = NULL;
    li->numItems = 0;
    li->currItem = 0;
    li->numSelected = 0;
    li->isActive = 0;
    li->userHasSetWidth = 0;
    li->startShowItem = 0;
    li->sbAdjust = 0;
    li->bdxAdjust = 0;
    li->bdyAdjust = 0;
    li->flags = flags & (NEWT_FLAG_RETURNEXIT | NEWT_FLAG_BORDER |
			 NEWT_FLAG_MULTIPLE);
    li->mctype = 0;

    if (li->flags & NEWT_FLAG_BORDER) {
	li->bdxAdjust = 2;
	li->bdyAdjust = 1;
    }
    co->height = height;
    li->curHeight = co->height - (2 * li->bdyAdjust);

    if (height) {
	li->grow = 0;
	if (flags & NEWT_FLAG_SCROLL) {
	    sb = newtVerticalScrollbar(left, top + li->bdyAdjust,
				       li->curHeight,
				  COLORSET_LISTBOX, COLORSET_ACTLISTBOX);
	    li->sbAdjust = 3;
	} else {
	    sb = NULL;
	}
    } else {
	li->grow = 1;
	sb = NULL;
    }

    li->sb = sb;
    co->data = li;
    co->isMapped = 0;
    co->left = left;
    co->top = top;
    co->ops = &checklistOps;
    co->takesFocus = 1;
    co->callback = NULL;

    updateWidth(co, li, 5);

    return co;
}

static inline void updateWidth(newtComponent co, struct checklist *li,
			       int maxField)
{
    li->curWidth = maxField;
    co->width = li->curWidth + li->sbAdjust + 2 * li->bdxAdjust;

    if (li->sb)
	li->sb->left = co->left + co->width - li->bdxAdjust - 1;
}

void newtChecklistSetCurrentByKey(newtComponent co, void *key)
{
    struct checklist *li = co->data;
    struct items *item;
    int i;

    item = li->boxItems, i = 0;
    while (item && item->data != key)
	item = item->next, i++;

    if (item)
	newtChecklistSetCurrent(co, i);
}

void newtChecklistSetCurrent(newtComponent co, int num)
{
    struct checklist *li = co->data;

    if (num >= li->numItems)
	li->currItem = li->numItems - 1;
    else if (num < 0)
	li->currItem = 0;
    else
	li->currItem = num;

    if (li->currItem < li->startShowItem)
	li->startShowItem = li->currItem;
    else if (li->currItem - li->startShowItem > co->height - 1)
	li->startShowItem = li->currItem - co->height + 1;
    if (li->startShowItem + co->height > li->numItems)
	li->startShowItem = li->numItems - co->height;
    if (li->startShowItem < 0)
	li->startShowItem = 0;

    newtChecklistRealSetCurrent(co);
}

static void newtChecklistRealSetCurrent(newtComponent co)
{
    struct checklist *li = co->data;

    if (li->sb)
	newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
    checklistDraw(co);
    if (co->callback)
	co->callback(co, co->callbackData);
}

void newtChecklistSetWidth(newtComponent co, int width)
{
    struct checklist *li = co->data;

    co->width = width;
    li->curWidth = co->width - li->sbAdjust - 2 * li->bdxAdjust;
    li->userHasSetWidth = 1;
    if (li->sb)
	li->sb->left = co->width + co->left - 1;
    checklistDraw(co);
}

void *newtChecklistGetCurrent(newtComponent co)
{
    struct checklist *li = co->data;
    int i;
    struct items *item;

    for (i = 0, item = li->boxItems; item != NULL && i < li->currItem;
	 i++, item = item->next);

    if (item)
	return (void *) item->data;
    else
	return NULL;
}


char newtChecklistGetCurrentValue(newtComponent co)
{
    struct checklist *li = co->data;
    int i;
    struct items *item;

    for (i = 0, item = li->boxItems; item != NULL && i < li->currItem;
	 i++, item = item->next);

    if (item)
	return item->isChecked;
    else
	return -1;
}

char newtChecklistGetCurrentValueByKey(newtComponent co, void *key)
{
    struct checklist *li = co->data;
    struct items *item;
    int i;

    item = li->boxItems, i = 0;
    while (item && item->data != key)
	item = item->next, i++;

    for (i = 0, item = li->boxItems; item != NULL &&
	 item->data != key; i++, item = item->next);

    if (item)
	return item->isChecked;
    else
	return -1;
}

void newtChecklistSelectItem(newtComponent co, const void *key,
			     enum newtFlagsSense sense)
{
    struct checklist *li = co->data;
    int i;
    struct items *item;

    item = li->boxItems, i = 0;
    while (item && item->data != key)
	item = item->next, i++;

    if (!item)
	return;

    if (item->isSelected)
	li->numSelected--;

    switch (sense) {
    case NEWT_FLAGS_RESET:
	item->isSelected = 0;
	break;
    case NEWT_FLAGS_SET:
	item->isSelected = 1;
	break;
    case NEWT_FLAGS_TOGGLE:
	item->isSelected = !item->isSelected;
    }

    if (item->isSelected)
	li->numSelected++;

    checklistDraw(co);
}

void newtChecklistClearSelection(newtComponent co)
{
    struct items *item;
    struct checklist *li = co->data;

    for (item = li->boxItems; item != NULL;
	 item = item->next)
	item->isSelected = 0;
    li->numSelected = 0;
    checklistDraw(co);
}

/* Free the returned array after use, but NOT the values in the array */
void **newtChecklistGetSelection(newtComponent co, int *numitems)
{
    struct checklist *li;
    int i;
    void **retval;
    struct items *item;

    if (!co || !numitems)
	return NULL;

    li = co->data;
    if (!li || !li->numSelected)
	return NULL;

    retval = malloc(li->numSelected * sizeof(void *));
    for (i = 0, item = li->boxItems; item != NULL;
	 item = item->next)
	if (item->isSelected)
	    retval[i++] = (void *) item->data;
    *numitems = li->numSelected;
    return retval;
}

void newtChecklistSetEntry(newtComponent co, int num, const char *text)
{
    struct checklist *li = co->data;
    int i;
    struct items *item;

    for (i = 0, item = li->boxItems; item != NULL && i < num;
	 i++, item = item->next);

    if (!item)
	return;
    else {
	free(item->text);
	item->text = strdup(text);
    }
    if (li->userHasSetWidth == 0 && strlen(text) > (size_t) li->curWidth) {
	updateWidth(co, li, strlen(text));
    }
    if (num >= li->startShowItem && num <= li->startShowItem + co->height)
	checklistDraw(co);
}

void newtChecklistSetData(newtComponent co, int num, void *data)
{
    struct checklist *li = co->data;
    int i;
    struct items *item;

    for (i = 0, item = li->boxItems; item != NULL && i < num;
	 i++, item = item->next);

    item->data = data;
}

void newtChecklistSetValue(newtComponent co, int num, int val)
{
    struct checklist *li = co->data;
    int i=0;
    struct items *item;

    for (item = li->boxItems; item != NULL && i < num;
	 i++, item = item->next);

    item->isChecked = val;
}

void newtChecklistSetValueByKey(newtComponent co, void *data, int val)
{
    struct checklist *li = co->data;
    struct items *item;

    for (item = li->boxItems; item != NULL && item->data != data;
	 item = item->next);

    item->isChecked = val;
}



int newtChecklistAppendEntry(newtComponent co, const char *text,
			     const void *data)
{
    return newtChecklistAppendCheckbox(co, text, -1, data);
}


int newtChecklistAppendCheckbox(newtComponent co, const char *text, int value,
				const void *data)
{
    struct checklist *li = co->data;
    struct items *item;

    if (li->boxItems) {
	for (item = li->boxItems; item->next != NULL; item = item->next);

	item = item->next = malloc(sizeof(struct items));
    } else {
	item = li->boxItems = malloc(sizeof(struct items));
    }

    if (!li->userHasSetWidth && text && (strlen(text) > (size_t) li->curWidth))
	updateWidth(co, li, strlen(text) + (value >= 0 ? 4 : 0));

    item->text = strdup(text);
    item->data = data;
    item->next = NULL;
    item->isSelected = 0;
    item->isChecked = value;
    item->ptrChecked = NULL;


    if (li->grow)
	co->height++, li->curHeight++;
    li->numItems++;

    return 0;
}

int newtChecklistInsertEntry(newtComponent co, const char *text,
			     const void *data, void *key)
{
    return newtChecklistInsertCheckbox(co, text, -1, data, key);
}

int newtChecklistInsertCheckbox(newtComponent co, const char *text,
				int value, const void *data, void *key)
{
    struct checklist *li = co->data;
    struct items *item, *t;

    if (li->boxItems) {
	if (key) {
	    item = li->boxItems;
	    while (item && item->data != key)
		item = item->next;

	    if (!item)
		return 1;

	    t = item->next;
	    item = item->next = malloc(sizeof(struct items));
	    item->next = t;
	} else {
	    t = li->boxItems;
	    item = li->boxItems = malloc(sizeof(struct items));
	    item->next = t;
	}
    } else if (key) {
	return 1;
    } else {
	item = li->boxItems = malloc(sizeof(struct items));
	item->next = NULL;
    }

    if (!li->userHasSetWidth && text && (strlen(text) > (size_t) li->curWidth))
	updateWidth(co, li, strlen(text) + (value >= 0 ? 4 : 0));

    item->text = strdup(text ? text : "(null)");
    item->data = data;
    item->isSelected = 0;
    item->isChecked = value;
    item->ptrChecked = NULL;

    if (li->sb)
	li->sb->left = co->left + co->width - li->bdxAdjust - 1;
    li->numItems++;

    checklistDraw(co);

    return 0;
}

int newtChecklistDeleteEntry(newtComponent co, void *key)
{
    struct checklist *li = co->data;
    int widest = 0, t;
    struct items *item, *item2 = NULL;
    int num;

    if (li->boxItems == NULL || li->numItems <= 0)
	return 0;

    num = 0;

    item2 = NULL, item = li->boxItems;
    while (item && item->data != key) {
	item2 = item;
	item = item->next;
	num++;
    }

    if (!item)
	return -1;

    if (item2)
	item2->next = item->next;
    else
	li->boxItems = item->next;

    free(item->text);
    free(item);
    li->numItems--;

    if (!li->userHasSetWidth) {
	widest = 0;
	for (item = li->boxItems; item != NULL; item = item->next)
	    if ((t = strlen(item->text)) > widest)
		widest = t;
    }
    if (li->currItem >= num)
	li->currItem--;

    if (!li->userHasSetWidth) {
	updateWidth(co, li, widest);
    }
    checklistDraw(co);

    return 0;
}

void newtChecklistClear(newtComponent co)
{
    struct checklist *li;
    struct items *anitem, *nextitem;
    if (co == NULL || (li = co->data) == NULL)
	return;
    for (anitem = li->boxItems; anitem != NULL; anitem = nextitem) {
	nextitem = anitem->next;
	free(anitem->text);
	free(anitem);
    }
    li->numItems = li->numSelected = li->currItem = li->startShowItem = 0;
    li->boxItems = NULL;
    if (!li->userHasSetWidth)
	updateWidth(co, li, 5);
}

/* If you don't want to get back the text, pass in NULL for the ptr-ptr. Same
   goes for the data. */
void newtChecklistGetEntry(newtComponent co, int num, char **text, void **data)
{
    struct checklist *li = co->data;
    int i;
    struct items *item;

    if (!li->boxItems || num >= li->numItems) {
	if (text)
	    *text = NULL;
	if (data)
	    *data = NULL;
	return;
    }
    i = 0;
    item = li->boxItems;
    while (item && i < num) {
	i++, item = item->next;
    }

    if (item) {
	if (text)
	    *text = item->text;
	if (data)
	    *data = (void *) item->data;
    }
}

void newtChecklistSetMCType(newtComponent co, int val)
{
    struct checklist *li = co->data;
    li->mctype = val;
}

void newtChecklistSetCheckboxPtr(newtComponent co, int index, char *ptr)
{
    struct checklist *li = co->data;
    struct items *item;
    int i;
    for (i = 0, item = li->boxItems; item != NULL && i < index;
	 i++, item = item->next);
    item->ptrChecked = ptr;
}

static void checklistDraw(newtComponent co)
{
    struct checklist *li = co->data;
    struct items *item;
    int i, j;

    if (!co->isMapped)
	return;

    if (li->flags & NEWT_FLAG_BORDER) {
	if (li->isActive)
	    SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
	else
	    SLsmg_set_color(NEWT_COLORSET_LISTBOX);

	newtDrawBox(co->left, co->top, co->width, co->height, 0);
    }
    if (li->sb)
	li->sb->ops->draw(li->sb);

    SLsmg_set_color(NEWT_COLORSET_LISTBOX);

    for (i = 0, item = li->boxItems; item != NULL && i < li->startShowItem;
	 i++, item = item->next);

    j = i;

    for (i = 0; item != NULL && i < li->curHeight; i++, item = item->next) {
	if (!item->text)
	    continue;

	newtGotorc(co->top + i + li->bdyAdjust, co->left + li->bdxAdjust);
	if (j + i == li->currItem) {
	    if (item->isSelected)
		SLsmg_set_color(NEWT_COLORSET_ACTSELLISTBOX);
	    else
		SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
	} else if (item->isSelected)
	    SLsmg_set_color(NEWT_COLORSET_SELLISTBOX);
	else {
	    if (li->mctype && item->isChecked != NEWT_CHECKLIST_SELECT_NOT) {
		if (item->isChecked == NEWT_CHECKLIST_SELECT_BLUE)
		    SLsmg_set_color(NEWT_COLORSET_LABEL);
	        else if (item->isChecked == NEWT_CHECKLIST_SELECT_GREEN)
		    SLsmg_set_color(NEWT_COLORSET_DISENTRY);
	        else if (item->isChecked == NEWT_CHECKLIST_SELECT_RED)
		    SLsmg_set_color(NEWT_COLORSET_ACTBUTTON);
		else
		    SLsmg_set_color(NEWT_COLORSET_LISTBOX);
	    }
	    else
		SLsmg_set_color(NEWT_COLORSET_LISTBOX);
	}

	if (!li->mctype) {
	    if (item->isChecked != NEWT_CHECKLIST_UNSELECTABLE) {
		SLsmg_printf("[%c] ", (item->isChecked ? 'X' : ' '));
		SLsmg_write_nstring(item->text, li->curWidth - 4);
	    } else
		SLsmg_write_nstring(item->text, li->curWidth);
	} else {
	    if (item->isChecked != NEWT_CHECKLIST_UNSELECTABLE) {
		char c1=' ',c2=' ';
		c1 = (item->isChecked == NEWT_CHECKLIST_SELECT_RED ? '<' : c1);
		c1 = (item->isChecked == NEWT_CHECKLIST_SELECT_BLUE ? '*' : c1);
		c1 = (item->isChecked == NEWT_CHECKLIST_SELECT_GREEN ? '$' : c1);
		c2 = (item->isChecked == NEWT_CHECKLIST_SELECT_RED ? '>' : c2);
		c2 = (item->isChecked == NEWT_CHECKLIST_SELECT_BLUE ? '*' : c2);
		c2 = (item->isChecked == NEWT_CHECKLIST_SELECT_GREEN ? '$' : c2);

		SLsmg_printf("%c",c1);
		SLsmg_write_nstring(item->text, li->curWidth - 2);
		SLsmg_printf("%c",c2);
	    } else
		SLsmg_write_nstring(item->text, li->curWidth);
	}

    }
    newtGotorc(co->top + (li->currItem - li->startShowItem), co->left);
}

enum newtChecklistStatus newtChecklistGetStatus(newtComponent co)
{
    struct checklist *li = co->data;
    return li->status;
}

void newtChecklistSetStatus(newtComponent co, enum newtChecklistStatus stat)
{
    struct checklist *li = co->data;
    li->status = stat;
}

static struct eventResult checklistEvent(newtComponent co, struct event ev)
{
    struct eventResult er;
    struct checklist *li = co->data;
    struct items *item;
    int i;

    er.result = ER_IGNORED;

    if (ev.when == EV_EARLY || ev.when == EV_LATE) {
	return er;
    }
    switch (ev.event) {
    case EV_KEYPRESS:
	if (!li->isActive)
	    break;
	li->status = NEWT_CHECKLIST_MOVE;
	switch (ev.u.key) {
	case ' ':
	    for (i = 0, item = li->boxItems; item != NULL && i < li->currItem;
		 i++, item = item->next);
	    if (item->isChecked != NEWT_CHECKLIST_UNSELECTABLE) {
		if (item->ptrChecked != NULL)
		    item->isChecked = item->ptrChecked[0];
		if (item->isChecked != NEWT_CHECKLIST_SELECT_GREEN)
 		    item->isChecked =  !item->isChecked;
		if (item->ptrChecked != NULL)
		    item->ptrChecked[0] = item->isChecked;
		checklistDraw(co);
		li->status = NEWT_CHECKLIST_SELECT;
		er.result = ER_SWALLOWED;
		if (!li->mctype)
		    break;
	    } else {
		if (!(li->flags & NEWT_FLAG_MULTIPLE))
		    break;
		newtChecklistSelectItem(co, li->boxItems[li->currItem].data,
					NEWT_FLAGS_TOGGLE);
		er.result = ER_SWALLOWED;
		/* We don't break here, because it is cool to be able to
		   hold space to select a bunch of items in a list at once */
	    }
	case NEWT_KEY_DOWN:
	    if (li->numItems <= 0)
		break;
	    if (li->currItem < li->numItems - 1) {
		li->currItem++;
		if (li->currItem > (li->startShowItem + li->curHeight - 1)) {
		    li->startShowItem = li->currItem - li->curHeight + 1;
		    if (li->startShowItem + li->curHeight > li->numItems)
			li->startShowItem = li->numItems - li->curHeight;
		}
		if (li->sb)
		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
		checklistDraw(co);
	    }
	    if (co->callback)
		co->callback(co, co->callbackData);
	    if (li->mctype && (li->flags & NEWT_FLAG_RETURNEXIT))
		er.result = ER_EXITFORM;
	    else
		er.result = ER_SWALLOWED;
	    break;

	case NEWT_KEY_ENTER:
	    if (li->numItems <= 0)
		break;
	    if (li->flags & NEWT_FLAG_RETURNEXIT)
		li->status = NEWT_CHECKLIST_ENTER;
	    er.result = ER_EXITFORM;
	    break;

	case NEWT_KEY_UP:
	    if (li->numItems <= 0)
		break;
	    if (li->currItem > 0) {
		li->currItem--;
		if (li->currItem < li->startShowItem)
		    li->startShowItem = li->currItem;
		if (li->sb)
		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
		checklistDraw(co);
	    }
	    if (co->callback)
		co->callback(co, co->callbackData);
	    if (li->mctype && (li->flags & NEWT_FLAG_RETURNEXIT))
		er.result = ER_EXITFORM;
	    else
		er.result = ER_SWALLOWED;
	    break;

	case NEWT_KEY_PGUP:
	    if (li->numItems <= 0)
		break;
	    li->startShowItem -= li->curHeight - 1;
	    if (li->startShowItem < 0)
		li->startShowItem = 0;
	    li->currItem -= li->curHeight - 1;
	    if (li->currItem < 0)
		li->currItem = 0;
	    newtChecklistRealSetCurrent(co);
	    if (li->mctype && (li->flags & NEWT_FLAG_RETURNEXIT))
		er.result = ER_EXITFORM;
	    else
		er.result = ER_SWALLOWED;
	    break;

	case NEWT_KEY_PGDN:
	    if (li->numItems <= 0)
		break;
	    li->startShowItem += li->curHeight;
	    if (li->startShowItem > (li->numItems - li->curHeight)) {
		li->startShowItem = li->numItems - li->curHeight;
	    }
	    li->currItem += li->curHeight;
	    if (li->currItem > li->numItems) {
		li->currItem = li->numItems - 1;
	    }
	    newtChecklistRealSetCurrent(co);
	    if (li->mctype && (li->flags & NEWT_FLAG_RETURNEXIT))
		er.result = ER_EXITFORM;
	    else
		er.result = ER_SWALLOWED;
	    break;

	case NEWT_KEY_HOME:
	    if (li->numItems <= 0)
		break;
	    newtChecklistSetCurrent(co, 0);
	    if (li->mctype && (li->flags & NEWT_FLAG_RETURNEXIT))
		er.result = ER_EXITFORM;
	    else
		er.result = ER_SWALLOWED;
	    break;

	case NEWT_KEY_END:
	    if (li->numItems <= 0)
		break;
	    li->startShowItem = li->numItems - li->curHeight;
	    if (li->startShowItem < 0)
		li->startShowItem = 0;
	    li->currItem = li->numItems - 1;
	    newtChecklistRealSetCurrent(co);
	    if (li->mctype && (li->flags & NEWT_FLAG_RETURNEXIT))
		er.result = ER_EXITFORM;
	    else
		er.result = ER_SWALLOWED;
	    break;
	default:
	    /* keeps gcc quiet */
	}
	break;

    case EV_FOCUS:
	li->isActive = 1;
	checklistDraw(co);
	er.result = ER_SWALLOWED;
	break;

    case EV_UNFOCUS:
	li->isActive = 0;
	checklistDraw(co);
	er.result = ER_SWALLOWED;
	break;
    }

    return er;
}

static void checklistDestroy(newtComponent co)
{
    struct checklist *li = co->data;
    struct items *item, *nextitem;

    nextitem = item = li->boxItems;

    while (item != NULL) {
	nextitem = item->next;
	free(item->text);
	free(item);
	item = nextitem;
    }

    free(li);
    free(co);
}
